/*
 * eth_ecat_pdo_fmmu.cpp
 *
 *  Created on: May 16, 2023
 *      Author: algin
 */

#include "free_rtos/ethernet_industry/CoE/eth_ecat_pdo_fmmu.hpp"

#include <algorithm>

namespace free_rtos {

namespace ecat_pdo_fmmu {

void EthEcatPdoFMMU::init(ProcessCallback callback) {
    std::vector<ecat_buffer::EcatBufferSlave>& buffer_slaves = ecat_buffer_.get_buffer_slaves();

    pdo_fmmu_slaves_.reserve(buffer_slaves.size());

    for(ecat_buffer::EcatBufferSlave& buffer_slave : buffer_slaves) {
        pdo_fmmu_slaves_.emplace_back(EcatPdoFMMUSlave{buffer_slave});
    }

    callback_ = callback;
}

void EthEcatPdoFMMU::process_write_queue(uint8_t* process_data, uint32_t len) {
    mutex_write_.lock();

    auto static_queue = static_queue_write_;
    auto queue = queue_write_;

    queue_write_.clear();

    mutex_write_.unlock();

    auto static_next = static_queue.dequeue();

    while(static_next != nullptr) {
        //DebugP_log("set value\r\n");
        static_next->set_value(process_data, len);
        //DebugP_log("queue read dequeue\r\n");
        static_next = static_queue.dequeue();
    }

    auto next = queue.dequeue();

    while(next != nullptr) {
        //DebugP_log("set value\r\n");
        next->set_value(process_data, len);
        //DebugP_log("queue write dequeue\r\n");
        next = queue.dequeue();
    }
}

void EthEcatPdoFMMU::process_read_queue(uint8_t* process_data, uint32_t len) {
    //DebugP_log("read lock\r\n");
    mutex_read_.lock();

    auto static_queue = static_queue_read_;
    auto queue = queue_read_;

    queue_read_.clear();

    mutex_read_.unlock();

    auto static_next = static_queue.dequeue();

    while(static_next != nullptr) {
        //DebugP_log("set value\r\n");
        static_next->set_value(process_data, len);
        //DebugP_log("queue read dequeue\r\n");
        static_next = static_queue.dequeue();
    }

    auto next = queue.dequeue();

    while(next != nullptr) {
        //DebugP_log("set value\r\n");
        next->set_value(process_data, len);
        //DebugP_log("queue read dequeue\r\n");
        next = queue.dequeue();
    }
}

}

}
