/*
 * eth_ecat_sdo_mailbox.cpp
 *
 *  Created on: May 16, 2023
 *      Author: algin
 */

#include "free_rtos/ethernet_industry/CoE/eth_ecat_sdo_mailbox.hpp"

namespace free_rtos {

namespace ecat_sdo_mailbox {

void EthEcatSdoMailbox::init() {
    std::vector<ecat_buffer::EcatBufferSlave>& buffer_slaves = ecat_buffer_.get_buffer_slaves();

    sdo_mailbox_slaves_.reserve(buffer_slaves.size());
    pdo_map_.reserve(buffer_slaves.size());

    for(ecat_buffer::EcatBufferSlave& mailbox_slave : buffer_slaves) {
        sdo_mailbox_slaves_.emplace_back(EcatSdoMailboxSlave{mailbox_slave});
    }
}

bool EthEcatSdoMailbox::read_pdo_map(uint16_t pdo_map_rx_index, uint16_t pdo_map_tx_index) {
    const datagram::TEcatWkc expected_wkc = 1;
    telegram::EcatTelegram& telegram = ecat_buffer_.get_ecat().get_telegram();
    bool status = true;

    for(EcatSdoMailboxSlave& sdo_mailbox_slave : sdo_mailbox_slaves_) {
        ecat_buffer::PDOMap pdo_map;

        DebugP_log((char*)"Reading rx pdo map\r\n");
        status = (status && sdo_mailbox_slave.pdo_map_read<command::FP>(telegram, expected_wkc, pdo_map, pdo_map_rx_index));

        DebugP_log((char*)"Reading tx pdo map\r\n");
        status = (status && sdo_mailbox_slave.pdo_map_read<command::FP>(telegram, expected_wkc, pdo_map, pdo_map_tx_index));

        pdo_map_.emplace_back(std::move(pdo_map));
    }

    return status;
}

}

}
