/*
 * eth_ecat_custom_tuple.hpp
 *
 *  Created on: May 5, 2023
 *      Author: algin
 */

#ifndef FREE_RTOS_ETHERNET_INDUSTRY_ETH_ECAT_CUSTOM_TUPLE_HPP_
#define FREE_RTOS_ETHERNET_INDUSTRY_ETH_ECAT_CUSTOM_TUPLE_HPP_

#include <vector>

namespace free_rtos {

// Базовый шаблон класса, никогда не инстанциируется, поэтому без тела
template<typename... Args>
struct custom_tuple;

// Основная специализация шаблона
template<typename HeadT, typename... TailT>
struct custom_tuple<HeadT, TailT...> : custom_tuple<TailT...> {
    custom_tuple(HeadT head, TailT... tail)
    : custom_tuple<TailT...>(tail...)
    , head_(head) { }

    custom_tuple()
    : custom_tuple<TailT...>() { }

    using TBase = custom_tuple<TailT...>;
    using THead = HeadT;
    using THeadDeref = typename std::remove_reference<THead>::type;

    constexpr static size_t size = sizeof(THeadDeref) + TBase::size;

    THead head_;
};

// Специализация завершения рекурсии
template<>
struct custom_tuple<> {
    constexpr static size_t size = 0;
};


// Базовый шаблон класса, никогда не инстанциируется, поэтому без тела
template<size_t index, typename TupleT>
struct custom_tuple_element;

// Основная специализация шаблона
// Наследоваться от custom_tuple_element< index - 1, custom_tuple<TailT...> > не обязательно
// т.к. используются только статические методы
template<size_t index, typename HeadT, typename... TailT>
struct custom_tuple_element< index, custom_tuple<HeadT, TailT...> > {
    using TTuple = custom_tuple<HeadT, TailT...>;
    using TBase = custom_tuple_element<index - 1, typename TTuple::TBase>;
    using type = typename TBase::type;

    static type& get(TTuple& t) {
        return TBase::get(t);
    }
};

// Специализация завершения рекурсии
template<typename HeadT, typename... TailT>
struct custom_tuple_element<0, custom_tuple<HeadT, TailT...>> {
    using TTuple = custom_tuple<HeadT, TailT...>;
    using type = HeadT;

    static type& get(TTuple& t) {
        return t.head_;
    }
};

// Функция получения элемента custom_tuple по индексу
template<size_t index, typename TupleT>
typename custom_tuple_element<index, TupleT>::type& get(TupleT& t) {
    return custom_tuple_element<index, TupleT>::get(t);
}


// Базовый шаблон класса, никогда не инстанциируется, поэтому без тела
template<typename FunctorT, typename TupleT>
struct each_tuple_element;

// Основная специализация шаблона. Есть еще одна в протоколе CoE SDO !
// Наследоваться от each_tuple_element< FunctorT, custom_tuple<TailT...> > не обязательно
// т.к. используются только статические методы
template<typename FunctorT, typename HeadT, typename... TailT>
struct each_tuple_element< FunctorT, custom_tuple<HeadT, TailT...> > {
    using TTuple = custom_tuple<HeadT, TailT...>;
    using TBase = each_tuple_element<FunctorT, typename TTuple::TBase>;

    static void for_each(FunctorT& functor, TTuple& t) {
        functor(t.head_);
        TBase::for_each(functor, t);
    }

    static void for_each_reverse(FunctorT& functor, TTuple& t) {
        TBase::for_each_reverse(functor, t);
        functor(t.head_);
    }
};

// Специализация завершения рекурсии
template<typename FunctorT>
struct each_tuple_element< FunctorT, custom_tuple<> > {
    using TTuple = custom_tuple<>;

    static void for_each(FunctorT& functor, TTuple& t) { }
    static void for_each_reverse(FunctorT& functor, TTuple& t) { }
};

// Функция для обхода элементов custom_tuple
template<typename TupleT, typename FunctorT>
void for_each(TupleT& t, FunctorT& functor) {
    each_tuple_element<FunctorT, TupleT>::for_each(functor, t);
}

// Функция для обратного обхода элементов custom_tuple
template<typename TupleT, typename FunctorT>
void for_each_reverse(TupleT& t, FunctorT& functor) {
    each_tuple_element<FunctorT, TupleT>::for_each_reverse(functor, t);
}

}

#endif /* FREE_RTOS_ETHERNET_INDUSTRY_ETH_ECAT_CUSTOM_TUPLE_HPP_ */
