/*
 * eth_ip_types.h
 *
 *  Created on: 13 мар. 2023 г.
 *      Author: sychev
 */

#ifndef FREE_RTOS_ETHERNET_IP_ETH_IP_TYPES_H_
#define FREE_RTOS_ETHERNET_IP_ETH_IP_TYPES_H_

#include <stdint.h>

#define ETH_IP_ADDR_LEN (4u)

#define ETH_PROT_IP_LE             0x0800 /**< 2048  (0x0800) IPv4 */
#define ETH_PROT_IP_BE             0x0008 /**< 2048  (0x0800) IPv4 */

typedef union {
    uint8_t bytes[4];
    uint32_t addr;
}TEthIpAddr;

/** ARP header structure */
typedef struct {
    uint16_t ar_hrd;   /**< Format of hardware address */
    uint16_t ar_pro;   /**< Format of protocol address */
    uint8_t ar_hln;    /**< Length of hardware address */
    uint8_t ar_pln;    /**< Length of protocol address */
    uint16_t ar_op;    /**< Operation */
    uint8_t ar_sha[6]; /**< Sender hardware address */
    uint8_t ar_spa[4]; /**< Sender protocol address */
    uint8_t ar_tha[6]; /**< Target hardware address */
    uint8_t ar_tpa[4]; /**< Target protocol address */
} __attribute__ ((packed)) TArpHeader;

/** IP Header structure */
typedef struct {
    uint8_t ip_hl_v;   /**< Header length and version */
    uint8_t ip_tos;    /**< Type of service */
    uint16_t ip_len;   /**< Total length */
    uint16_t ip_id;    /**< Identification */
    uint16_t ip_off;   /**< Fragment offset field */
    uint8_t ip_ttl;    /**< Time to live */
    uint8_t ip_p;      /**< Protocol */
    uint16_t ip_sum;   /**< Checksum */
    uint8_t ip_src[4]; /**< Source IP address */
    uint8_t ip_dst[4]; /**< Destination IP address */
} __attribute__ ((packed)) TIpHeader; /* GCC */


#endif /* FREE_RTOS_ETHERNET_IP_ETH_IP_TYPES_H_ */
